package info1.ships;
import  info1.ships.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * une implémentation "abstraite" d'un bateau quelconque, de taille indéterminée
 */


public abstract class Ship implements IShip {

    private String name;
    Coord cord1;
    Coord cord2;


    /**
     * NB : LA SIGNATURE DU CONSTRUCTEUR DOIT ETRE RESPECTEE
     *
     * construit un bateau quelconque
     * @param name le nom du bateau
     * @param ayFront la coordonnée de la proue du bateau
     * @param ayBack la coordonnée de la poupe du bateau
     * @throws BadCoordException si l'une des coordonnées données ne définit pas une coordonnée alphanumérique correcte
     * @throws CoordsBadShipException si les coordonnées données ne permettent pas de définir un bateau correct :
     *  une ligne, une colonne, de la bonne taille, etc.
     */
    public Ship(String name, String ayFront, String ayBack)
            throws BadCoordException, CoordsBadShipException {
        this.name=name;
        this.cord1=new Coord(ayFront);
        this.cord2=new Coord(ayBack);
    }


    @Override
    public List<ICoord> getCoords() {
        ArrayList<ICoord> list = new ArrayList<>();
        list.add(cord1);
        list.add(cord2);
        return list;
    }

    @Override
    public ICoord getFront() {
        return cord1;
    }

    @Override
    public ICoord getBack() {
        return cord2;
    }

    @Override
    public String getName() {
        return this.name;
    }



    @Override
    public int getSize() {
        if(cord1.getY()==cord2.getY()){
            if(cord1.getX()>cord2.getX())
                return cord1.getX()-cord2.getX()+1;
            return cord2.getX()-cord1.getX()+1;
        }
        if(cord1.getX()==cord2.getX()){
            if(cord1.getY()>cord2.getY())
                return cord1.getY()-cord2.getY()+1;
            return cord2.getY()-cord1.getY()+1;
        }
        return -1;
    }

    @Override
    public String toString() {
        if (cord1.toString().equals(cord2.toString()))
            return this.name + cord1.toString();
        return this.name + cord1.toString() + cord2.toString();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Ship ship = (Ship) o;
        return Objects.equals(name, ship.name) && Objects.equals(cord1, ship.cord1) && Objects.equals(cord2, ship.cord2);
    }

    @Override
    public int hashCode() {
        return Objects.hash(name, cord1, cord2);
    }
}
